#include "iodefine.h"
#include "DebugCom.h"
#include "ADC.h"


#define	MVPERVALUE 3.22 

unsigned int meassuredData[16];



//Initialiserer den valgte ADC modulen.
void initADCmodule(int moduleNumber)
{
	switch (moduleNumber)
	{
		case 0:
			//Setter modulen ut av stand-by.
			MST.CR2.BIT._AD0 = 0;
	
			//Setter konverteringshastighet (P/32) (s385) (OBS!! Prvde P/4, men det ser ut som dette delegger resultatet)
			AD0.ADCR.BIT.CKS = 0x00;
	
			//Ellers: .ADST har initval 0 som betyr at modulen kjrer single-cycle scan.
	
			//Velger 8-channel scan mode (s. 383)
			AD0.ADCSR.BIT.ADM = 2;
	
			//Velger  mle spenning mellom AN0-AN3,AN8-AN11 og AD4-AN7,AN12-AN15 (s. 384)
			AD0.ADCSR.BIT.CH   = 3;
	
			//Starter ADC-modulen
			AD0.ADCR.BIT.ADST = 1;
			break;
		case 1:
			//Setter modulen ut av stand-by.
			MST.CR2.BIT._AD1 = 0;
	
			//Setter konverteringshastighet (P/32) (s385) (OBS!! Prvde P/4, men det ser ut som dette delegger resultatet)
			AD1.ADCR.BIT.CKS = 0x00;
	
			//Ellers: .ADST har initval 0 som betyr at modulen kjrer single-cycle scan.
	
			//Velger 8-channel scan mode (s. 383)
			AD1.ADCSR.BIT.ADM = 2;
	
			//Velger  mle spenning mellom AN0-AN3,AN8-AN11 og AD4-AN7,AN12-AN15 (s. 384)
			AD1.ADCSR.BIT.CH   = 3;
	
			//Starter ADC-modulen
			AD1.ADCR.BIT.ADST = 1;
			break;
	}
}


void updateMeassuredData(int channel, int newValue)
{
	if( (abs(newValue-meassuredData[channel]) > MINVALUECHANGE) )
	{
		meassuredData[channel] = newValue;	
	}
}


//Starter kovertering i valgt modul, venter p resultat og lagrer resultatet i meassureData[].
//OBS! Dersom man starter en konvertering p en modul som ikke er initialisert kan(vil?) funksjonen
//henge seg opp i while-lkka!
void startSingleScan(int moduleNumber)
{
	switch (moduleNumber)
	{
		case 0:
			AD0.ADCR.BIT.ADST = 1;
			while(!AD0.ADCR.BIT.ADST);
			meassuredData[0] = AD0.ADDR0.WORD>>6;
			meassuredData[1] = AD0.ADDR1.WORD>>6;
			meassuredData[2] = AD0.ADDR2.WORD>>6;
			meassuredData[3] = AD0.ADDR3.WORD>>6;
			meassuredData[8] = AD0.ADDR8.WORD>>6;
			meassuredData[9] = AD0.ADDR9.WORD>>6;
			meassuredData[10] = AD0.ADDR10.WORD>>6;
			meassuredData[11] = AD0.ADDR11.WORD>>6;	
			break;
		case 1:
		 	AD1.ADCR.BIT.ADST = 1;
			while(!AD1.ADCR.BIT.ADST);
			meassuredData[4] = AD1.ADDR4.WORD>>6;
			meassuredData[5] = AD1.ADDR5.WORD>>6;
			meassuredData[6] = AD1.ADDR6.WORD>>6;
			meassuredData[7] = AD1.ADDR7.WORD>>6;
			meassuredData[12] = AD1.ADDR12.WORD>>6;
			meassuredData[13] = AD1.ADDR13.WORD>>6;
			meassuredData[14] = AD1.ADDR14.WORD>>6;
			meassuredData[15] = AD1.ADDR15.WORD>>6;
			break;
	}	
}

//Samme som over, men filtrer ut mlesty. Filtreringen kan tunes ved hjelp av variabelen MINVALUECHANGE som finnes i ADC.h
void startSingleScanWithFilter(int moduleNumber)
{
	switch (moduleNumber)
	{
		case 0:
			AD0.ADCR.BIT.ADST = 1;
			while(!AD0.ADCR.BIT.ADST);
			updateMeassuredData(0,AD0.ADDR0.WORD>>6);
			updateMeassuredData(1,AD0.ADDR1.WORD>>6);
			updateMeassuredData(2,AD0.ADDR2.WORD>>6);
			updateMeassuredData(3,AD0.ADDR3.WORD>>6);
			updateMeassuredData(8,AD0.ADDR8.WORD>>6);
			updateMeassuredData(9,AD0.ADDR9.WORD>>6);
			updateMeassuredData(10,AD0.ADDR10.WORD>>6);
			updateMeassuredData(11,AD0.ADDR11.WORD>>6);
			break;
		case 1:
		 	AD1.ADCR.BIT.ADST = 1;
			while(!AD1.ADCR.BIT.ADST);
			updateMeassuredData(4,AD1.ADDR4.WORD>>6);
			updateMeassuredData(5,AD1.ADDR5.WORD>>6);
			updateMeassuredData(6,AD1.ADDR6.WORD>>6);
			updateMeassuredData(7,AD1.ADDR7.WORD>>6);
			updateMeassuredData(12,AD1.ADDR12.WORD>>6);
			updateMeassuredData(13,AD1.ADDR13.WORD>>6);
			updateMeassuredData(14,AD1.ADDR14.WORD>>6);
			updateMeassuredData(15,AD1.ADDR15.WORD>>6);
			break;
	}	
}





unsigned int readChannel(int channel)
{
	return meassuredData[channel];	
}

unsigned int getmVoltage(int channel)
{
	return meassuredData[channel]*MVPERVALUE;
}

void printAllChannels(void)
{
		pStrWNum("Channel_0: %d\r\n",readChannel(0));
		pStrWNum("Channel_1: %d\r\n",readChannel(1));
		pStrWNum("Channel_2: %d\r\n",readChannel(2));
		pStrWNum("Channel_3: %d\r\n",readChannel(3));
		pStrWNum("Channel_4: %d\r\n",readChannel(4));
		pStrWNum("Channel_5: %d\r\n",readChannel(5));
		pStrWNum("Channel_6: %d\r\n",readChannel(6));
		pStrWNum("Channel_7: %d\r\n",readChannel(7));
		pStrWNum("Channel_8: %d\r\n",readChannel(8));
		pStrWNum("Channel_9: %d\r\n",readChannel(9));
		pStrWNum("Channel_10: %d\r\n",readChannel(10));
		pStrWNum("Channel_11: %d\r\n",readChannel(11));
		pStrWNum("Channel_12: %d\r\n",readChannel(12));
		pStrWNum("Channel_13: %d\r\n",readChannel(13));
		pStrWNum("Channel_14: %d\r\n",readChannel(14));
		pStrWNum("Channel_15: %d\r\n",readChannel(15));	
}