#include "iodefine.h"
#include "DebugCom.h"
#include "GlobVar.h"



unsigned char StringWithNumber[20]; 				//String som brukes til  lagre stringer med tall som skal sendes over seriellporten.
unsigned char VariableDebugString[20] = "NEI";		//String som brukes til  sende valgfri meldinger til debug-blokken fra hvor som helst i koden.
unsigned char inputFromComputer;					//Her lagres eventuelle bytes som sendes fra datamaskinen.
unsigned char newInputFromComputerReady = FALSE;

void initDebugCom(void)
{
	unsigned char speed;
	
	// Sets function of PA3 as RXD3.
	PFC.PACRL2.BIT.PA3MD = 0x02;
	PFC.PACRL3.BIT.PA3MD = 0x01;
	
	// Sets function of PA4 as TXD3.
	PFC.PACRL2.BIT.PA4MD = 0x02;
	PFC.PACRL3.BIT.PA4MD = 0x01;
	
	// Sets PA3 as input pin.
	PFC.PAIORL.BIT.B3 = 0x00;
	
	// Sets PA4 ;as output pin.
	PFC.PAIORL.BIT.B4 = 0x01;
	
	// Clears the standby mode for the SCI3 module.
	MST.CR1.BIT._SCI3 = 0;
	
	// Sets interrupt priority register.
	INTC.IPRI.BIT._SCI3 = SCI3_INT_PRIORITY;
	
	// Sets the serial control register.
	// Disable transmit interrupt, enable receive interrupt, transmit and receive enabled.
	SCI3.SCR.BYTE = 0x70;
	
	// Sets the serial mode register.
	// Asynchronous mode, 8 bits data length, parity disabled, 1 stop bit, n = 0 (baud rate parameter).
	SCI3.SMR.BYTE = 0x00;
	
	// Sets the BBR register according to the selected speed.
	// Example: Freq=22MHz, bps=115200->n=0,N=5 BRR=5
	speed = (unsigned char) (( ((double)peripheralClock / (double)32) / (double)SCI3_BAUDRATE + 0.5) - 1.0);
	SCI3.BRR = speed;
	
	pStr("*************************INITIALISERING START************************\r\n");
	pStr("Seriell kommunikasjon er initialisert.\r\n");
}

void writeChar(char c)
{
	// Loops until the transmit data register is empty.
	while(!SCI3.SSR.BIT.TDRE);
	// Loads the data byte into the transmit data register.
	SCI3.TDR = c;
	// Clears the TDRE bit to indicate that the transmit data register is not empty.
	SCI3.SSR.BIT.TDRE = 0;
}


/* PSTR
 * Sender en vanlig string over seriellporten.
 */
void pStr(char* s)
{
	short int i;
	
	// Writes each valid character in the array. The loop is ended when the
	// NULL character (0) is encountered.
	for(i=0; s[i]; i++)
	{
		writeChar(s[i]);
	}
}

/* PSTRWNUM
 * Samme som over, men kan i tillegg legge til en variabel.
 */
void pStrWNum(char* s,int number){
	sprintf(StringWithNumber,s,number);
	pStr(StringWithNumber);

}

void saveRecievedByte(unsigned char recievedByte)
{
	if(!newInputFromComputerReady) //Unng overskriving
	{
		inputFromComputer = recievedByte;
		newInputFromComputerReady = TRUE;
		SCI3.SSR.BIT.RDRF = 0; //Forteller mikrokontrolleren at byten er lest.
	}
}

unsigned char getInputFromComputer(void)
{
	newInputFromComputerReady = FALSE;
	return inputFromComputer;
}

unsigned char newInputReady(void){
	return newInputFromComputerReady;
}

void cleanTerminal(void)
{
	int i;
	for(i = 0; i<10; i++)
	{
		pStr("\r\n");
	}
}

void sendDataToLabview(int weight, int ADCvalue, int Angle)
{
	sprintf(StringWithNumber,"W%05dA%05dV%05d\n",weight, ADCvalue, Angle);
	pStr(StringWithNumber);	
}

void setVariableDebugMessage(char* s, int number)
{
	sprintf(VariableDebugString,s,number);
}

void printVariableDebugMessage(void)
{
	pStr(VariableDebugString);
}	