#include "iodefine.h"
#include "PWM.h"
#include "DebugCom.h"
#include "Angle.h"
#include "Servo.h"


//PWM-periode: 20ms
//PWM-pulsbredde: 0.9-2.1 ms

//Utregning av periode. nsker en 20ms periode for styring av servo. P =  22 118 400 Hz
//Med P/16-prescaler vil counter_0 telle 1 382 400 ganger i sekundet.
//Det vil si at (1 382 400/50 = )27 648 vil gi en periode p 20 ms. (27 648 = 0x6C00)
//Videre tilsvarer 1 244 (0x04DC) omtrent 0,9 ms og 2 903 (0x0B57) omtrent 2,1 ms.

#define PWMperiod 	27648
#define PWMpulseCenter 2073

#define COUNT_PER_MICROSECOND 1.3824 	//Hvor mange tellinger som utfres i lpet av ett mikrosekund.

void initPWM(void)
{
	
	//Setter PE1 (TIOC0B) som output.
	PFC.PECRL2.BIT.PE0MD = 1;
	PFC.PEIORL.BIT.B0 = 1;
	
	//Setter MTU-modulen ut av stand-by
	MST.CR2.BIT._MTU = 0;
	
	
	//Setter opp PWM etter Figur 10.20 (s. 209)
	MTU0.TCR.BIT.TPSC = 0x02; //Prescaler P/16. (se side 158 i datablad)
	MTU0.TCR.BIT.CKEG = 0x00; //Tell klokke p stigende kant.
	
	MTU0.TCR.BIT.CCLR = 0x01; //TCNT cleared by TGRA compare match (se side 157 i datablad)
	
	MTU0.TIOR.BIT.IOA = 0x02; //TGRA Init output 0, output at compare match 1. (s.165)
	MTU0.TIOR.BIT.IOB = 0x05; //TGRB Init output 1, output at compare match 0. (s.164)
	

	MTU0.TGRA = PWMperiod; //Lengden p perioden til PWM-signalet. 
	MTU0.TGRB = PWMpulseCenter; //Duty-cycle[%] = (1 - TGRB/TGRA)*100.
	
	MTU0.TMDR.BIT.MD = 0x02; //PWM mode 1 (s.161).

	MTU.TSTR.BYTE = 0x01; //Starter counter 1 (s.186).	
	
	setServoSpeed(0); //Roer ned servoen i tilfelle den er litt aggressiv.
	pStr("PWM er initialisert.\r\n");
}


//Setter PWM-signalet til en bredde mellom 900 og 2100 mikrosekund.
void setPWMpulseWidth(int microseconds)
{
	//Srger for at verdien ligger mellom 900 og 2100:
	if(microseconds>2100)
		microseconds = 2100;
	else if(microseconds<900)
		microseconds = 900;
		
	int countToThis = microseconds*COUNT_PER_MICROSECOND;
	MTU0.TGRB = countToThis;
}
	