#include "Servo.h"
#include "PWM.h"
#include "Angle.h"
#include "Timer.h"


#define ZEROSPEED_PULSEWIDTH 1513				//Pulsbredden p PWM-signalet som lar servoen st i ro. Gitt i mikrosekund.
#define MAX_DEVIATION_FROM_ZEROSPEED 50			//Antall mikrosekund som kontrolleren fr avvike fra midtpunktet. Denne br vre satt til det ytterpunktet som gir metning i fart.


//Parameters
int desiredAngle = 0;				//nsket vinkelposisjon
int angleError = 0;					//Differansen mellom nsket vinkel og faktisk vinkel.
int angleErrorOld = 0;
int errorDiff = 0;

//PID parameters
int Kp = 3;
int Ki = 0;
int Kd = 1;



/*	SETSERVOSPEED
 *	Denne funksjonen styrer farten til servoen. Variabelen 'percent' kan ligge mellom -100 og 100.
 *	Farten stilles til en prosent av maksfarten (som er bestemt av 'MAX_DEVIATION_FROM_ZEROSPEED') 
 *	Retningen bestemmes av fortegnet til 'percent', alts str servoen i ro nr verdien '0' settes.
 */
 
int setServoSpeed(int percent)
{
	int newPulseWidth; //Gitt i us.
	double percentDouble; //Gjr det mulig  dele p 100.
	percentDouble = (double)percent;
	
	//Srger frst for at percent ligger mellom -100 og 100:
	if(percentDouble>100)
		percentDouble = 100;
	else if(percentDouble<-100)
		percentDouble = -100;
		
	newPulseWidth = ZEROSPEED_PULSEWIDTH+(MAX_DEVIATION_FROM_ZEROSPEED*(-percentDouble/100));
	setPWMpulseWidth(newPulseWidth);
	return newPulseWidth;
}

/*	SETDESIREDANGLE
 *	Denne funksjonen setter verdien p 'desiredAngle'
 */
void setDesiredAngle(int angle)
{
	//Srger frst for at angle ligger mellom -80 og 80:
	if(angle>80)
		angle = 80;
	else if(angle<-80)
		angle = -80;
	desiredAngle = angle;
}

int getDesiredAngle(void)
{
	return desiredAngle;
}

int getAngleError(void)
{
	return angleError;
}

void updateAngleError(void)
{
	angleErrorOld = angleError;
	angleError = desiredAngle-getAngle();
	errorDiff = angleError-angleErrorOld;
}


/* UPDATEPID
 * Her oppdateres PID-regulatoren.
 */
int updatePID(void)
{
	int PWMpulsewidth;
	updateAngleError();
	PWMpulsewidth = setServoSpeed(Kp*angleError-Kd*errorDiff);
	return PWMpulsewidth;
	
}