#include "iodefine.h"
#include "Timer.h"
#include "DebugCom.h"
#include "GlobVar.h"


void initTimerInt(void)
{	
	csPassed = 0;	//Setter centisekundteller til 0.
	
	periodicLoopCounter = 0;
	
	unsigned int compMatch = 0xFFFF;
		
	// Clears the standby mode for the Timer CTM1 and CMT0.
	MST.CR2.BIT._CMT=0;
	// Stops CMT counter 1.
	CMT.CMSTR.BIT.STR1 = 0;
	// Enables compare match interrupts.
	CMT1.CMCSR.BIT.CMIE = 1;
	// Sets the counting frequency as the peripheral clock (P) / 512. (p.399)
	// P = 22 118 400 Hz => Counting frequency = 43 200 Hz
	CMT1.CMCSR.BIT.CKS = 0x03;
	// Resets the counter to zero.
	CMT1.CMCNT = 0;
	// Sets the counter value of CMCNT which will generate a compare match interrupt each centisecond.
	// With a counting frequency of 43 200 Hz the compare match interrupt has to happen when 
	// CMCNT = 43200/1000 =432
	CMT1.CMCOR = 432;
	// Set to zero to set a "NOT MATCHED CONDITION".
	CMT1.CMCSR.BIT.CMF &= 0;
	// Sets the interrupt priority.
	INTC.IPRG.BIT._CMT1 = 0x09;
	// Starts CMT counter 1.
	CMT.CMSTR.BIT.STR1 = 1;
	pStr("Timer er initialisert\r\n");
	
}


/* delay_cs
 * Vanlig delayfunksjon. Operer i centisekund.
 */
void delay_cs(int cs)
{
	unsigned int startTime = csPassed;
	while( (startTime+cs)>csPassed){
		//Wait
		}
}


/* doPeriodicTasks
 * Brukes til  time kjring av periodiske oppgaver i main-lkka.
 */
unsigned char doPeriodicTasks(void)
{
	if(periodicLoopCounter > PERIODIC_LOOP_LIMIT)
	{
		periodicLoopCounter = 0;
		//pStr("Peridoc do? YES!\r\n");
		return TRUE;
	}
	//pStr("Peridoc do? NO!\r\n");
	return FALSE;
}


/* csSinceLastRun
 * Returnerer hvor mange centisekund som har passert siden sist funksjonen ble kjrt.
 */
int lastTimeStamp = 0;
int csSinceLastRun(void)
{	
	int csSinceLast;
	csSinceLast = csPassed-lastTimeStamp;
	lastTimeStamp = csPassed;
	return csSinceLast;
} 
	
	